@@ -19,6 +19,7 @@ PUBLIC_SOURCES = "public_sources"
 PUBLIC_DESTINATIONS = "public_destinations"
 UPLOAD_ENDPOINT = "https://api.bayse.io/stats"
 API_KEY_DEFAULT_ENVIRONMENT_VARIABLE = "BAYSE_API_KEY"
+MAX_FILE_SIZE_KB = 200  # a bit less than a backend limit, to avoid missing data
 
 
 def collect_data_from_bayseflow_files(filepaths, verbose):
@@ -29,13 +30,23 @@ def collect_data_from_bayseflow_files(filepaths, verbose):
     collected_bayseflow_fields = ["src", "dst", "relativeStart", "duration", "protocolInformation", "label"]
     for p in filepaths:
         path = pathlib.PurePath(p)
-        if not re.match(EXPECTED_FILETYPE_REGEX, magic.from_file(path)):
+        valid_filetype = re.match(EXPECTED_FILETYPE_REGEX, magic.from_file(path))
+        data = None
+        if not valid_filetype:  # re.match(EXPECTED_FILETYPE_REGEX, magic.from_file(path)):
             if verbose:
-                print(f"{path} is not a JSON file. Skipping.")
-            continue
-        else:
+                print(f"{path} is not a JSON file. Trying to load anyway.")
+            try:
+                with open(path, "r") as bfdata:
+                    data = json.load(bfdata)
+                    valid_filetype = True
+            except:
+                if verbose:
+                    print(f"Failed to load {path}. Skipping.")
+                continue
+        if valid_filetype:
             with open(path, "r") as bfdata:
-                data = json.load(bfdata)
+                if not data:
+                    data = json.load(bfdata)
                 try:
                     first_bayseflow = data[BAYSEFLOWS_IDENTIFIER][0]
                     file_start_time = data["trafficDate"]
@@ -90,6 +101,7 @@ def collect_bayseflow_data_by_direction(summarized_data, absolute_start, baysefl
     flowend = flowstart + float(bayseflow["duration"])
     label_details["first_observed"] = min(label_details["first_observed"], flowstart)
     label_details["last_observed"] = max(label_details["last_observed"], flowend)
+    return len(protocol_dict) / 1024  # rough calculation of KB size of entry
 
 
 def summarize_bayseflow_files(filepaths, verbose=False):
@@ -127,6 +139,9 @@ def summarize_bayseflow_files(filepaths, verbose=False):
     """Now that we have all of the data we really need, go through all of the collected data and summarize how many 
        times we've seen each flow that is North-South or South-North. Ignore the others.
     """
+    data_size_kb = 0
+    saved_files = []
+    file_count = 1
     for contents in relevant_bayseflow_data:
         absolute_start = float(contents["file_start_time"])
         if absolute_start == DEFAULT_TIME:
@@ -203,14 +218,30 @@ def summarize_bayseflow_files(filepaths, verbose=False):
                             continue
                 # anything global that is still processing at this point should be captured for summarization.
                 if global_source:
-                    collect_bayseflow_data_by_direction(summarized_data, absolute_start, bayseflow, str(source),
-                                                        source_port, PUBLIC_SOURCES, verbose)
+                    data_size_kb += collect_bayseflow_data_by_direction(summarized_data, absolute_start, bayseflow,
+                                                                        str(source), source_port, PUBLIC_SOURCES,
+                                                                        verbose)
                 if global_destination:
-                    collect_bayseflow_data_by_direction(summarized_data, absolute_start, bayseflow,
-                                                        str(destination), destination_port, PUBLIC_DESTINATIONS,
-                                                        verbose)
+                    data_size_kb += collect_bayseflow_data_by_direction(summarized_data, absolute_start, bayseflow,
+                                                                        str(destination), destination_port,
+                                                                        PUBLIC_DESTINATIONS, verbose)
+                if data_size_kb > MAX_FILE_SIZE_KB:
+                    saved_files += [save_stats_file(summarized_data, file_count, verbose)]  # save current file now
+                    data_size_kb = 0  # reset KB count
+                    summarized_data = {PUBLIC_SOURCES: dict(),
+                                       PUBLIC_DESTINATIONS: dict()
+                                       }  # reset these too
+                    file_count += 1
             # we implicitly ignore summarizing E-W data
-    summarized_data_location = pathlib.PurePath(f"{str(time.time()).split('.')[0]}_summary.json")  # timestamped files
+    saved_files += [save_stats_file(summarized_data, file_count, verbose)]
+    return saved_files
+
+
+def save_stats_file(summarized_data, file_count, verbose=False):
+    """Temporarily saves a stats file. Returns the location of the file. File_count var helps to make sure we can
+       capture when we actually need to save multiple to fit within backend limitations.
+    """
+    summarized_data_location = pathlib.PurePath(f"{str(time.time()).split('.')[0]}_{file_count}_summary.json")  # ts
     with open(summarized_data_location, "w") as out:
         json.dump(summarized_data, out)
         if verbose:
@@ -218,13 +249,15 @@ def summarize_bayseflow_files(filepaths, verbose=False):
     return summarized_data_location
 
 
-def upload_stats(summarized_data_location, api_key=None, api_key_environment_variable=None):
+def upload_stats(summarized_data_locations, api_key=None, api_key_environment_variable=None):
     """Send the stats data to the server for a given user. Expects API key (i.e. the user's identifier) to be either
        passed in, to have the name of an environment variable, or to be in the default environment value location. If
        none of these exist, upload fails. Otherwise success is dependent on whether the data is accepted server-side.
     """
     upload_info = {"status": "Failed", "errors": "", "text": ""}
-    location = pathlib.PurePath(summarized_data_location)
+    locations = []
+    for data_loc in summarized_data_locations:
+        locations += [pathlib.PurePath(data_loc)]
     summary_data = None
     if api_key is None and api_key_environment_variable is None:
         api_key = os.environ.get(API_KEY_DEFAULT_ENVIRONMENT_VARIABLE)
@@ -238,38 +271,48 @@ def upload_stats(summarized_data_location, api_key=None, api_key_environment_var
                                      f" {api_key_environment_variable}.\n"
     if api_key is None:
         upload_info["errors"] += f"No API key found.\n"
-        pathlib.Path(location).unlink(missing_ok=True)
+        for location in locations:
+            pathlib.Path(location).unlink(missing_ok=True)
         return upload_info  # short-circuit here
 
-    if not re.match(EXPECTED_FILETYPE_REGEX, magic.from_file(location)):
-        try:  # Some OSes fail to capture JSON as JSON, so try to load data. Server-side validates data passed to it.
-            with open(location, "r") as summary_datafile:
-                summary_data = json.load(summary_datafile)
-        except Exception as e:
-            upload_info["errors"] += f"{location} is not a JSON file.\n"
-    else:
-        try:
-            with open(location, "r") as summary_datafile:
-                summary_data = json.load(summary_datafile)
-        except Exception as e:
-            upload_info["errors"] += f"{e}\n"
-    if not summary_data:
-        upload_info["errors"] += f"No data found when trying to open and load {summarized_data_location}.\n"
-    elif PUBLIC_SOURCES not in summary_data or PUBLIC_DESTINATIONS not in summary_data:
-        upload_info["errors"] += f"{PUBLIC_SOURCES} and/or {PUBLIC_DESTINATIONS} missing from " \
-                                 f"{summarized_data_location}\n"
-    else:  # the remainder of error cases needs to be handled server-side, so prep the upload
-        headers = {"X-API-KEY": api_key}
-        response = requests.request("POST", url=UPLOAD_ENDPOINT, headers=headers, json=summary_data)
-        if not response.ok:
-            upload_info["errors"] += f"{response.text}\n"
+    s = requests.Session()
+    for location in locations:
+        if not re.match(EXPECTED_FILETYPE_REGEX, magic.from_file(location)):
+            try:  # Some OSes fail to capture JSON as JSON, so try to load data. Server-side validates data passed to it
+                with open(location, "r") as summary_datafile:
+                    summary_data = json.load(summary_datafile)
+            except Exception as e:
+                upload_info["errors"] += f"{location} is not a JSON file.\n"
+                continue
         else:
-            upload_info["status"] = f"Success"
-            upload_info["text"] = response.text
-    pathlib.Path(location).unlink(missing_ok=True)
+            try:
+                with open(location, "r") as summary_datafile:
+                    summary_data = json.load(summary_datafile)
+            except Exception as e:
+                upload_info["errors"] += f"{e}\n"
+                continue
+        if not summary_data:
+            upload_info["errors"] += f"No data found when trying to open and load {location}.\n"
+            continue
+        elif PUBLIC_SOURCES not in summary_data or PUBLIC_DESTINATIONS not in summary_data:
+            upload_info["errors"] += f"{PUBLIC_SOURCES} and/or {PUBLIC_DESTINATIONS} missing from {location}\n"
+        else:  # the remainder of error cases needs to be handled server-side, so prep the upload
+            headers = {"X-API-KEY": api_key}
+            # Prepare to use a session
+            req = requests.Request("POST", url=UPLOAD_ENDPOINT, headers=headers, json=summary_data)
+            prepped = req.prepare()
+            response = s.send(prepped)
+            if not response.ok:
+                upload_info["errors"] += f"{response.text}\n"
+            else:
+                upload_info["status"] = f"Success"
+                upload_info["text"] = response.text
+        pathlib.Path(location).unlink(missing_ok=True)
     return upload_info
 
 
 if __name__ == "__main__":
-    summarize_bayseflow_files(["../../tests/valid_file1.bf", "../../tests/valid_file2.bf", "../../tests/valid_file3.bf",
-                               "../../tests/invalid_file1.bf"], verbose=True)
+    outputs = summarize_bayseflow_files(["../../tests/valid_file1.bf", "../../tests/valid_file2.bf",
+                                        "../../tests/valid_file3.bf",
+                                        "../../tests/invalid_file1.bf"], verbose=True)
+    print(f"Saved files at {outputs}")
